/* hslcom.c: read in hue, sat, and luminosity values and combine */ 
/*  into a single r,g,b image in TGA format. */
/*           John P. Beale 6/10/95                           */
/*          GIF library routines from gd library             */

#include <stdio.h>                      /* fopen() sprintf() etc. */
#include <stdlib.h>                     /* strtod() */
#include <math.h>                       /* math stuff */
#include <string.h>                     /* strcpy() */
#include "hcon.h"
#include "hf_io.h"

#define PIXTYPE unsigned char           /* TGA: single byte pixel plane */

int tga3_write(Cimg imout, char *fname);
void HSL_to_RGB(D h,D sl,D l,D *r,D *g,D *b);

U xsize, ysize;     /* size of hf array (input xsize+2) */
U xsize2, ysize2;
U xsize3, ysize3;
Cimg imout;                      /* 3-color-plane image map */
PTYPE *im1, *im2, *im3;                      /* height field array */
PTYPE fmin = 0, fmax = 1.0;                 /* scaling limits */
Boolean norenorm = FALSE;
Boolean negate = FALSE;
Boolean rescale = FALSE;
Boolean clip = FALSE;


/* ---------------------------------- */

int main(int argc, char **argv)
{ char *usage;
  char *utemp = 
"hslcom v0.1a A HSL to RGB converter (C) <beale@jump.stanford.edu> 1995\n\
Usage: hslcom <H> <S> <L> [hmin hmax smin smax lmin lmax]\n\
       where H, S, and L are input hue, sat, luminosity filenames. \n\
       Optionally specify H,S,L ranges, which default to full [0...1]. \n\
       Writes output file hsl.tga which is the combined R-G-B image.\n\
       All three input files must be of the same dimension.\n\
       Input file type may be PGM,GIF,POT,MAT,TGA (all grayscale formats)\n\
       example usage:   hslcom hue.pgm sat.pgm lum.pgm\n";
  
  char fname1[80], fname2[80], fname3[80];
  char fname_out[160] = "hsl.tga";
  char *buf;                         /* dummy char. buffer */
  int rval;
  int ix, iy; /* iteration vars */
  int i;
  D r,g,b;    /* real-valued r,g,b components */
  D lims[6];

  usage = (char *)malloc((size_t)(1000 * sizeof(char)));
  sprintf(usage,utemp,Intype_string, Outtype_string);

  if ((argc < 4) || (argc > 10)) {
	fprintf(stderr,"%s",usage);
	exit(1);
      }
 
  strcpy(fname1,argv[1]);
  strcpy(fname2,argv[2]);
  strcpy(fname3,argv[3]);
  i = 4;
  lims[0] = lims[2] = lims[4] = 0.0;  /* H, S, L limits */
  lims[1] = lims[3] = lims[5] = 1.0;
  if (argc > 4) {
    do {
      lims[i-4] = strtod(argv[i],&buf);
      i++;
    } while (i < argc);
  }
  printf("H [%1.2f-%1.2f]  S [%1.2f-%1.2f]  L [%1.2f-%1.2f]\n",
	lims[0],lims[1],lims[2],lims[3],lims[4],lims[5]);

					/* read in heightfield */
   rval = readn_hf(&im1, &xsize, &ysize, fname1);
   if (rval != 0)
     exit(rval);
   rval = readn_hf(&im2, &xsize2, &ysize2, fname2);
   if (rval != 0)
     exit(rval);
   rval = readn_hf(&im3, &xsize3, &ysize3, fname3);
   if (rval != 0)
     exit(rval);

   printf("%s: [%d x %d] \n",fname1,xsize,ysize);
   printf("%s: [%d x %d] \n",fname2,xsize2,ysize2);
   printf("%s: [%d x %d] \n",fname3,xsize3,ysize3);
   if (xsize2!=xsize3 || xsize2!= xsize || ysize2!=ysize3 || ysize2!=ysize) {
     fprintf(stderr,"Error: input image dimensions not equal.\n");
     exit(1);
   }
   imout.xsize = xsize;
   imout.ysize = ysize;
   imout.r = (PIXTYPE *) malloc(sizeof(PIXTYPE)*xsize*ysize);
   imout.g = (PIXTYPE *) malloc(sizeof(PIXTYPE)*xsize*ysize);
   imout.b = (PIXTYPE *) malloc(sizeof(PIXTYPE)*xsize*ysize);
   if (imout.r==NULL || imout.g==NULL || imout.b==NULL) {
	fprintf(stderr,"Out of memory for TGA output file.\n");
	exit(1);
   }
   for (ix=0;ix<xsize;ix++) {
     for (iy=0;iy<ysize;iy++) {
	HSL_to_RGB(     ( El(im1,ix,iy) * (lims[1]-lims[0]))+lims[0],
			(El2(im2,ix,iy) * (lims[3]-lims[2]))+lims[2],
			(El3(im3,ix,iy) * (lims[5]-lims[4]))+lims[4],
			&r,&g,&b);
	Elr(imout,ix,iy) = (PIXTYPE)(r*255.99);
	Elg(imout,ix,iy) = (PIXTYPE)(g*255.99);
	Elb(imout,ix,iy) = (PIXTYPE)(b*255.99);
     }
   }

   rval = tga3_write(imout, fname_out);

   return(rval);    /* return a value */
} /* end main() */

